%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: July 2016
%
%   References:
%   P. N. Alevizos, A. Bletsas, "Network Localization Cramer-Rao
%   Bounds for General Measurement Models," IEEE Communications Letters,
%   Volume: 20, Issue: 9, Sept. 2016 , pp. 1840--1843, 2016.
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**


clear all;
clc;

%a predetermined 3D topology (you can also use you own)
load agents_v9
load anchors_v9


%concatenate agent and anchor nodes
nodes = [agents anchors ] ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%N + L : number of nodes
N_L = length(nodes);
N = length(agents);

%euclidean distance between nodes
% d = dist(nodes', nodes)
%for matlab 2015
d = pdist2(nodes',nodes');

%% RSS parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%path loss exponent (PLE) (assuming common PLE accross all measurements)
nu = 2.3;
%standard deviation of error
sigma_RSS = [ 2 : 1 : 16];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% ToA parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%propagation velocity
prop_velocity = 3*10^8;
%standard deviation of error
sigma_ToA = [3 : 1 : 12] * 10^(-9); %(seconds)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Communication radius(m) (>8 for invertible FIM)
radius = 15 + 0.0*randn(N_L);

%Crammer - Rao bounds for differen combinations of measurements
%RSS
crb_RSSno_direct = zeros(1,length(sigma_RSS));
crb_RSSdirect = crb_RSSno_direct;
crb_RSSdirect_ToA = crb_RSSno_direct;

%ToA
crb_ToA = zeros(1,length(sigma_ToA));
crb_ToA_RSSno_direct = crb_ToA;
crb_ToA_RSSdirect = crb_ToA;



%%           FIM evaluation for diferent measurement models
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% NxN matrix where unity indicates connected nodes
A = zeros(N_L,N_L); A(d<=radius & d~=0) = 1;

%% RSS
disp('3D  RSS')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for sigma_count = 1 : length(sigma_RSS)
    
    sigma_ToA_tmp = sigma_ToA(4); %(=6 nano seconds)
    
    %Fisher information calculation according to Eq. (16)
    J_RSSno_direct = FIM_RSS_3D( A, N, N_L, sigma_RSS(sigma_count), nu, nodes, d );
    J_RSS_direct =  FIM_RSS_dipoles_3D( A, N, N_L, sigma_RSS(sigma_count), nu, nodes, d );
    J_ToA = FIM_ToA_3D( A, N, N_L, sigma_ToA_tmp, prop_velocity, nodes, d );
    J_RSSdirect_ToA = J_ToA +  J_RSS_direct;
    
    %CRB
    crb_RSSno_direct(sigma_count) = trace(inv(J_RSSno_direct))/N;
    crb_RSSdirect(sigma_count) = trace(inv(J_RSS_direct))/N;
    crb_RSSdirect_ToA(sigma_count) = trace(inv(J_RSSdirect_ToA))/N;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% ToA
disp('3D ToA')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for sigma_count = 1 : length(sigma_ToA)
    
    sigma_RSS_tmp = sigma_RSS(3); %(=4 dB)
    
    %Fisher information calculation according to Eq. (16)
    J_RSSno_direct = FIM_RSS_3D( A, N, N_L, sigma_RSS_tmp, nu, nodes, d );
    J_RSS_direct =  FIM_RSS_dipoles_3D( A, N, N_L, sigma_RSS_tmp, nu, nodes, d );
    J_ToA = FIM_ToA_3D( A, N, N_L, sigma_ToA(sigma_count), prop_velocity, nodes, d );
    J_ToA_RSSdirect = J_ToA +  J_RSS_direct;
    J_ToA_RSSno_direct = J_ToA +  J_RSSno_direct;

    %CRB
    crb_ToA(sigma_count) = trace(inv(J_ToA))/N;
    crb_ToA_RSSno_direct(sigma_count) = trace(inv(J_ToA_RSSno_direct))/N;
    crb_ToA_RSSdirect(sigma_count) = trace(inv(J_ToA_RSSdirect))/N;
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% plots
close all;

%RSS
semilogy(sigma_RSS, sqrt(crb_RSSno_direct), 'g-+', 'Linewidth', 2,'Markersize', 13)
hold on;
semilogy(sigma_RSS, sqrt(crb_RSSdirect), 'b-o', 'Linewidth', 2, 'Markersize', 13)
semilogy(sigma_RSS, sqrt(crb_RSSdirect_ToA), 'k:', 'Linewidth', 3)
hold off
xlabel('\sigma^{(2)} (dB)')
ylabel('RMSE')
legend('RSS', 'RSS with dipoles', 'RSS+ToA(\sigma^{(1)} = 6 nano sec.)')

%ToA
figure;
semilogy(sigma_ToA, sqrt(crb_ToA),'c-d', 'Linewidth', 2,'Markersize', 13)
hold on;
semilogy(sigma_ToA, sqrt(crb_ToA_RSSno_direct),'k:', 'Linewidth', 3,'Markersize', 13)
semilogy(sigma_ToA, sqrt(crb_ToA_RSSdirect), 'm:x', 'Linewidth', 3,'Markersize', 13)
hold off
xlabel('\sigma^{(1)} (seconds)')
ylabel('RMSE')
legend('ToA', 'ToA + RSS(\sigma^{(2)} = 4 dB)', 'ToA + RSS with dipoles(\sigma^{(2)} = 4 dB)')


